@@ -1,13 +1,13 @@
-# Copyrights 2004,2006-2014 by [Mark Overmeer].
+# Copyrights 2004,2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use strict;
 use warnings;
 
 package Math::Polygon::Calc;
 use vars '$VERSION';
-$VERSION = '1.03';
+$VERSION = '1.05';
 
 use base 'Exporter';
 
@@ -15,32 +15,33 @@ our @EXPORT = qw/
  polygon_area
  polygon_bbox
  polygon_beautify
+ polygon_centroid
+ polygon_clockwise
+ polygon_contains_point
+ polygon_counter_clockwise
+ polygon_distance
  polygon_equal
  polygon_is_clockwise
  polygon_is_closed
- polygon_clockwise
- polygon_counter_clockwise
  polygon_perimeter
  polygon_same
  polygon_start_minxy
  polygon_string
- polygon_contains_point
- polygon_centroid
 /;
 
 use List::Util    qw/min max/;
 use Carp          qw/croak/;
 
 
-sub polygon_string(@) { join ', ', map { "[$_->[0],$_->[1]]" } @_ }
+sub polygon_string(@) { join ', ', map "[$_->[0],$_->[1]]", @_ }
 
 
 sub polygon_bbox(@)
 {
-    ( min( map {$_->[0]} @_ )
-    , min( map {$_->[1]} @_ )
-    , max( map {$_->[0]} @_ )
-    , max( map {$_->[1]} @_ )
+    ( min( map $_->[0], @_ )
+    , min( map $_->[1], @_ )
+    , max( map $_->[0], @_ )
+    , max( map $_->[1], @_ )
     );
 }
 
@@ -242,7 +243,7 @@ sub polygon_same($$;$)
 
 
 # Algorithms can be found at
-# http://astronomy.swin.edu.au/~pbourke/geometry/insidepoly/
+# http://www.eecs.umich.edu/courses/eecs380/HANDOUTS/PROJ2/InsidePoly.html
 # p1 = polygon[0];
 # for (i=1;i<=N;i++) {
 #   p2 = polygon[i % N];
@@ -275,10 +276,17 @@ sub polygon_contains_point($@)
 
     while(@_)
     {   my ($nx, $ny) = @{ (shift) };
+
+        # Extra check for exactly on the edge when the axes are
+        # horizontal or vertical.
         return 1 if $y==$py && $py==$ny
                  && ($x >= $px || $x >= $nx)
                  && ($x <= $px || $x <= $nx);
 
+        return 1 if $x==$px && $px==$nx
+                 && ($y >= $py || $y >= $ny)
+                 && ($y <= $py || $y <= $ny);
+
         if(   $py == $ny
            || ($y <= $py && $y <= $ny)
            || ($y >  $py && $y >  $ny)
@@ -289,8 +297,10 @@ sub polygon_contains_point($@)
             next;
         }
 
+        # side wrt diagonal
+        my $xinters = ($y-$py)*($nx-$px)/($ny-$py)+$px;
         $inside = !$inside
-            if $px==$nx || $x <= ($y-$py)*($nx-$px)/($ny-$py)+$px;
+            if $px==$nx || $x <= $xinters;
 
         ($px, $py) = ($nx, $ny);
     }
@@ -323,4 +333,51 @@ sub polygon_is_closed(@)
     $first->[0]==$last->[0] && $first->[1]==$last->[1];
 }
 
-1;
+
+# Contributed by Andreas Koenig for 1.05
+# http://stackoverflow.com/questions/10983872/distance-from-a-point-to-a-polygon#10984080
+# with correction from
+# http://stackoverflow.com/questions/849211/shortest-distance-between-a-point-and-a-line-segment
+sub polygon_distance($%)
+{   my $p = shift;
+
+    my ($x, $y) = @$p;
+    my $minDist;
+
+    @_ or return undef;
+
+	my ($x1, $y1) = @{ (shift) };
+	unless(@_)
+	{   my ($dx, $dy) = ($x1 - $x, $y1 - $y);
+        return sqrt($dx * $dx + $dy * $dy);
+    }
+
+    while(@_)
+    {   my ($x2, $y2) = @{ (shift) };   # closed poly!
+        my $A =  $x - $x1;
+        my $B =  $y - $y1;
+        my $C = $x2 - $x1;
+        my $D = $y2 - $y1;
+
+        # closest point to the line segment
+        my $dot    = $A * $C + $B * $D;
+        my $len_sq = $C * $C + $D * $D;
+        my $angle  = $len_sq==0 ? -1 : $dot / $len_sq;
+ 
+        my ($xx, $yy)
+         = $angle < 0 ? ($x1, $y1)   # perpendicular line crosses off segment
+         : $angle > 1 ? ($x2, $y2)
+         :              ($x1 + $angle * $C, $y1 + $angle * $D);
+
+        my $dx = $x - $xx;
+        my $dy = $y - $yy;
+        my $dist = sqrt($dx * $dx + $dy * $dy);
+        $minDist = $dist unless defined $minDist;
+        $minDist = $dist if $dist < $minDist;
+
+		($x1, $y1) = ($x2, $y2);
+    }
+
+    $minDist;
+}
+
